from django.urls import path
from rest_framework.views import View
from rest_framework.viewsets import ViewSet

from drf_decorator_router.base import BaseRouter


class Router(BaseRouter):
    def process_viewset(self, viewset: ViewSet, route: str, basename: str) -> None:
        pass

    def process_view(self, view: View, route: str, name: str) -> None:
        pass

    def route_viewset(self, route: str, basename: str = ""):
        def inner(viewset):
            self._log(
                "Routing viewset " + str(viewset) + " to " + self.endpoint + route
            )
            self.process_viewset(viewset, route, basename)
            self._router.register(route, viewset, basename=basename)
            return viewset

        return inner

    def route_view(self, route: str, name: str = ""):
        def inner(view):
            self._log("Routing view " + str(view) + " to " + self.endpoint + route)
            self.process_view(view, route, name)
            self._urls.append(path(route, view.as_view(), name=name))
            return view

        return inner



from rest_framework import viewsets, generics
from rest_framework.response import Response

from .routers import router

from . import models, serializers


@router.route_viewset("mock-store", "mock-store")
class MockStoreViewSet(viewsets.ModelViewSet):
    queryset = models.MockStore.objects.all()
    serializer_class = serializers.MockStoreSerializer


@router.route_view("mock-products/", "mock-products")
class MockProductsListView(generics.ListAPIView):
    queryset = models.MockProduct.objects.all()
    serializer_class = serializers.MockProductSerializer


@router.route_view("mock-store/<int:store_id>/mock-products/", "mock-store-products")
class MockStoreProductsListView(MockProductsListView):
    def list(self, request, *args, **kwargs):
        queryset = self.queryset.filter(store__id=kwargs.get('store_id'))
        serializer = self.serializer_class(queryset, many=True)

        return Response(serializer.data)



import importlib
from inspect import isclass

from rest_framework.routers import DefaultRouter
from rest_framework.viewsets import ViewSet
from rest_framework.views import View

from django.urls import path, include
from django.conf import settings

from .utils import clean_app_name, get_modules_list


class BaseRouter:
    _auto_load_modules = True

    def __init__(
        self, endpoint: str, namespace: str, module: str = None, debug: bool = False
    ) -> None:
        self.endpoint: str = endpoint
        self.namespace: str = namespace
        self.module: str = module
        self._router = DefaultRouter()
        self._urls = []
        self.debug = debug


    def _log(self, message: str) -> None:
        if self.debug:
            print("Auto Router - " + message)

    def _load_all_modules(self) -> None:
        for module in settings.INSTALLED_APPS:
            try:
                self._log("Searching for views in app " + clean_app_name(module))
                for module_name in get_modules_list():
                    module = importlib.import_module(
                        clean_app_name(module) + "." + module_name
                    )
                    self._log("Found module " + str(module))
                    for attribute_name in dir(module):
                        attribute = getattr(module, attribute_name)

                        if isclass(attribute):
                            # Add the class to this package's variables
                            globals()[attribute_name] = attribute
            except ModuleNotFoundError:
                pass
            except Exception as e:
                raise e


    @property
    def urls(self) -> list[str]:
        return [*self._urls, *self._router.urls]

    @property
    def path(self) -> path:
        if self._auto_load_modules:
            if self.module:
                importlib.import_module(self.module)
            else:
                self._load_all_modules()

            self._log("Done loading views")

        return path(
            self.endpoint,
            include((self.urls, "main"), namespace=self.namespace),
        )



# Generated by Django 4.2.8 on 2023-12-24 20:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MockStore',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=30)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MockProduct',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=30)),
                ('store', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mockapp.mockstore')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from collections import namedtuple

from django.http import HttpResponse
from django.urls import path
from rest_framework.fields import Field
from rest_framework.generics import ListAPIView
from rest_framework.response import Response
from rest_framework.serializers import Serializer
from rest_framework.views import View
from rest_framework.viewsets import ViewSet

from drf_decorator_router.router import Router

NUMBER_FIELDS = ['IntegerField', 'FloatField', 'DecimalField']
STRING_FIELDS = ['CharField', 'TextField', 'SlugField', 'UUIDField', 'DateField', 'DateTimeField']
LIST_SERIALIZER = "ListSerializer"

INTERFACE_TEMPLATE = """
export interface {{name}}{
    {{fields}}
}
"""

BaseInterface = namedtuple("BaseInterface", ["name", "fields", "order"])


class BaseTranslator:
    interfaces: set[BaseInterface] = set()
    serializers: set[Serializer] = set()
    registered_serializers: set[str] = set()

    def get_interface_name(self, serializer: Serializer) -> str:
        return serializer.__class__.__name__.replace("Serializer", "")

    def get_interface_name_from_field(self, field: Field) -> str:
        return str(field).split("(")[0].replace("Serializer", "")

    def get_field_type(self, field: Field) -> str:
        name = field.__class__.__name__

        if name in NUMBER_FIELDS:
            return 'number'
        elif name in STRING_FIELDS:
            return 'string'
        elif name == LIST_SERIALIZER:
            return self.get_interface_name_from_field(field)

        return "unknown"

    def register_serializer(self, serializer: Serializer):
        name = self.get_interface_name(serializer)

        if name in self.registered_serializers:
            return

        fields = list(map(lambda f: f'{f[0]}: {self.get_field_type(f[1])}', serializer.get_fields().items()))
        self.registered_serializers.add(name)
        self.interfaces.add(BaseInterface(name, "\n".join(fields), 0))

    def interface_to_string(self, interface: BaseInterface) -> str:
        return INTERFACE_TEMPLATE.replace("{{name}}", interface.name).replace("{{fields}}", interface.fields)

    def register_view(self, view: View | ViewSet):
        view_instance = view()
        serializer = view_instance.get_serializer_class()()

        self.serializers.add(serializer)

    def get_typescript_file(self) -> str:
        for serializer in self.serializers:
            self.register_serializer(serializer)

        return "\n".join(map(lambda i: self.interface_to_string(i) ,self.interfaces))


class TSTypingsView(ListAPIView):
    translator: BaseTranslator

    def list(self, request, *args, **kwargs):
        return HttpResponse(
            self.translator.get_typescript_file(),
            content_type="text/javascript",
        )


class TSRouter(Router):
    translator = BaseTranslator()

    def process_viewset(self, viewset: ViewSet, route: str, basename: str) -> None:
        self.translator.register_view(viewset)

    def process_view(self, view: View, route: str, name: str) -> None:
        self.translator.register_view(view)

    def on_views_loaded(self) -> None:
        TSTypingsView.translator = self.translator

        self._urls.append(
            path("typings/", TSTypingsView.as_view(), name="django-ts-typings")
        )



from django.conf import settings


def clean_app_name(app_name) -> str:
    split = str(app_name).split(".")
    end_index = len(split)

    for index, element in enumerate(split):
        if index == 0:
            continue

        if element == "apps":
            end_index = index
            break

    return ".".join(split[0:end_index])


def get_modules_list() -> list[str]:
    if hasattr(settings, "AUTO_ROUTER_MODULES"):
        return getattr(settings, "AUTO_ROUTER_MODULES")

    return ["views"]



from .admin_router import AdminRouter
from .base import BaseRouter
from .router import Router
from .tsrouter import TSRouter

__exports__ = [
    Router,
    TSRouter,
    AdminRouter,
    BaseRouter
]



from rest_framework.test import APITestCase
from rest_framework.reverse import reverse

from model_mommy import mommy

from .mockapp.models import MockProduct, MockStore


class DecoratorRouterAPITestCase(APITestCase):
    store_list_url = reverse("api-v1:mock-store-list")

    def setUp(self):
        self.store_1 = mommy.make(MockStore)
        self.store_2 = mommy.make(MockStore)

        for store in [self.store_1, self.store_2]:
            mommy.make(MockProduct, store=store, _quantity=10)

    def test_it_lists_stores_from_viewset(self):
        response = self.client.get(self.store_list_url)
        json = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(json), 2)

    def test_it_retrieves_store_from_viewset(self):
        url = reverse("api-v1:mock-store-detail", (self.store_1.id,))
        response = self.client.get(url)
        json = response.json()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(json['id'], self.store_1.id)


from django.apps import AppConfig


class MockappConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'tests.mockapp'

    def ready(self):
        from . import routers



from rest_framework import serializers

from . import models


class MockProductSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.MockProduct
        exclude = []


class MockStoreSerializer(serializers.ModelSerializer):
    products = MockProductSerializer(source="mockproduct_set", many=True)

    class Meta:
        model = models.MockStore
        exclude = []




from django.contrib.admin import ModelAdmin, site
from django.db.models import Model
from rest_framework.permissions import IsAdminUser
from rest_framework.serializers import ModelSerializer
from rest_framework.viewsets import ModelViewSet

from .base import BaseRouter


class ModelAdminSerializer(ModelSerializer):
    class Meta:
        fields = []


class AdminRouter(BaseRouter):
    def register(self, model: Model, route: str, basename: str = ""):
        def inner(model_admin: ModelAdmin):
            site.register(model, model_admin)
            self._log(
                "Routing admin viewset "
                + str(model_admin)
                + " to "
                + self.endpoint
                + route
            )

            class ModelAdminViewSet(ModelViewSet):
                model_admin: ModelAdmin
                permission_classes = [IsAdminUser]

                def get_queryset(self):
                    return self.model_admin.get_queryset(self.request)

                def get_serializer_class(self):
                    cls = ModelAdminSerializer
                    cls.Meta.model = self.model_admin.model
                    cls.Meta.fields = self.model_admin.list_display

                    print(dir(self.model_admin.get_changelist_form(self.request)))
                    print(
                        self.model_admin.get_changelist_form(self.request).base_fields
                    )

                    return cls

            viewset = ModelAdminViewSet
            viewset.model_admin = model_admin(model, site)
            self._router.register(route, viewset, basename=basename)
            return viewset

        return inner



from django.contrib import admin

from . import models
from .routers import admin_router


@admin_router.register(models.MockProduct, "mock-product")
class MockProductAdmin(admin.ModelAdmin):
    list_display = ("id", "name", "store")


@admin_router.register(models.MockStore, "mock-store")
class MockStoreAdmin(admin.ModelAdmin):
    list_display = ("id", "name")



# -*- coding: utf-8 -*-

# read the contents of your README file
from pathlib import Path

from setuptools import setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="drf-decorator-router",
    version="0.3",
    description="Fast API like decorator for routing DRF Views and Viewsets.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Breno da Costa Ribeiro Gomes",
    author_email="brenodega28@gmail.com",
    url="https://github.com/brenodega28/drf-decorator-router",
    download_url="https://github.com/brenodega28/drf-decorator-router.git",
    license="MIT License",
    packages=[
        "drf_decorator_router",
    ],
    include_package_data=True,
    install_requires=[
        "Django>=2.2,<4.3",
        "djangorestframework>=3.7,<=3.14",
    ],
    tests_require=[
        "model-mommy",
    ],
    zip_safe=False,
    classifiers=[
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.6"
)



from django.db import models


class TimeStampedModel(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class MockStore(TimeStampedModel):
    name = models.CharField(max_length=30)

    def __str__(self) -> str:
        return self.name


class MockProduct(TimeStampedModel):
    name = models.CharField(max_length=30)
    store = models.ForeignKey(MockStore, on_delete=models.CASCADE)

    def __str__(self) -> str:
        return f"({self.store.name}) - {self.name}"
